from Products.GrufSpaces import grufspaces_globals
from StringIO import StringIO
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.DirectoryView import addDirectoryViews
from Acquisition import aq_base
from OFS.Folder import manage_addFolder
from Products.CMFCore.CMFCorePermissions import setDefaultRoles
from Products.GrufSpaces.global_symbols import *
from Products.ExternalMethod.ExternalMethod import ExternalMethod

SKIN_NAME = "grufspaces"
_globals = globals()


GroupSpace_workflows = (
        ('groupspace_open_close_workflow', ('GroupSpace'), 0),
        )

def install_plone(self, out):
    pass

def install_subskin(self, out, skin_name=SKIN_NAME, globals=grufspaces_globals):
    print >>out, "  Installing subskin."
    skinstool=getToolByName(self, 'portal_skins')
    if skin_name not in skinstool.objectIds():
        print >>out, "    Adding directory view for GRUF"
        addDirectoryViews(skinstool, 'skins', globals)

    for skinName in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skinName) 
        path = [i.strip() for i in  path.split(',')]
        try:
            if skin_name not in path:
                path.insert(path.index('custom') +1, skin_name)
        except ValueError:
            if skin_name not in path:
                path.append(skin_name)  

        path = ','.join(path)
        skinstool.addSkinSelection( skinName, path)
    print >>out, "  Done installing subskin."

def walk(out, obj, operation):
    if obj.isPrincipiaFolderish:
        for content in obj.objectValues():
            walk(out, content, operation)
    operation(out, obj)


def setupCustomSlots(self, out):
   """ sets up the custom slots"""
   slots = (
       'here/portlet_groups/macros/portlet',
       'here/portlet_groupspaces_changes/macros/portlet',
       )
   for slot in slots: 
       if hasattr(self, "right_slots"):
           if slot in self.right_slots:
               out.write("Right slot exists already\n")
           else:
               right_slots_list=list(self.right_slots)
               self._delProperty('right_slots')
               right_slots_list.append(slot)
               self._setProperty('right_slots',tuple(right_slots_list),'lines')
               out.write("Added %s slot\n" % (slot, ))

def install(self):
    out = StringIO()
    print >>out, "Installing GroupUserFolder"
    install_subskin(self, out)
    install_plone(self, out)
    print >>out, "Done."

    from Products.GrufSpaces.Installation import Installation
    installation = Installation(self)
    if installation.isPlone2():
        print >>out, "Installing GroupSpace features"
        from Products.GrufSpaces import GroupSpace

        installation.setupTypesandSkins((GroupSpace.factory_type_information,), "", grufspaces_globals)

        if not "GroupVisitor" in installation.root.userdefined_roles():
            installation.root._addRole("GroupVisitor")
        if not "GroupMember" in installation.root.userdefined_roles():
            installation.root._addRole("GroupMember")
        if not "GroupAdmin" in installation.root.userdefined_roles():
            installation.root._addRole("GroupAdmin")
        if not "GroupLeader" in installation.root.userdefined_roles():
            installation.root._addRole("GroupLeader")

        catalog = getToolByName(self, 'portal_catalog')
        if not "listGroupSpaceMemberUsers" in catalog.indexes():
            print >>out, "Adding GroupSpace-related indexes"
            catalog.manage_addIndex('listGroupSpaceMemberUsers', 'KeywordIndex', )
        if not "getGroupSpaceId" in catalog.indexes():
            catalog.manage_addIndex('getGroupSpaceId', 'FieldIndex', )
        wf_tool = installation.root.portal_workflow
        
        # DEPRECATED
        #if not "groupspace_workflow" in wf_tool.objectIds():
        #    print >>out, "Installing GroupSpace-specific Workflow"
        #    wf_tool._importObjectFromFile(GroupSpace.DATA_PATH, verify = 1, set_owner = 1)
        #    wf_tool.setChainForPortalTypes(('GroupSpace',), 'groupspace_workflow')
        
        
        # OPEN CLOSE WORKFLOW
        # Adding the worflows

        for name, types, default in GroupSpace_workflows:
            if name not in wf_tool.objectIds():
                installFunc = ExternalMethod('temp', 'temp', 'GrufSpaces' + '.' + name,
                                             'create' + name.capitalize())
                workflow = installFunc(name)
                wf_tool._setObject(name, workflow)
                if types:
                    wf_tool.setChainForPortalTypes(types, name)
                if default:
                    wf_tool.setDefaultChain(name)
        
        
        installation.root.portal_groups.setGroupWorkspaceType("GroupSpace")
        setupCustomSlots(self, out)
        
        # register GroupSpace in use_folder_contents
        props = self.portal_properties.site_properties
        use_folder_tabs = list(props.use_folder_tabs)
        if 'GroupSpace' not in use_folder_tabs:
            use_folder_tabs.append('GroupSpace')
            props.use_folder_tabs=tuple(use_folder_tabs)
        
        print >>out, installation.report()
        print >>out, "Done."

    return out.getvalue()
    
def uninstall(self):
    out = StringIO()
    
    from Products.GrufSpaces.Installation import Installation
    installation = Installation(self)
    
    if installation.isPlone2():
        wf_tool = installation.root.portal_workflow
        # uninstall workflows
        
        for workflow_id, types, default in GroupSpace_workflows:
            if workflow_id in wf_tool.objectIds():
                wf_tool._delObject(workflow_id)
